@@ -2,13 +2,12 @@ import os
 import argparse
 import itertools
 from ete3 import Tree
+from Bio import AlignIO
 
 
 Dating_usage = '''
 ============================= Dating example commands =============================
 
-Dependencies:
-
 # example commands
 TreeSAK Dating -deltall DeltaLL_stdout.txt -aod s11_marker_sets_by_DeltaLL -o s12_dating_wd -c 25-50-75-100 -mmn 20 -f
 
@@ -44,9 +43,9 @@ def root_with_out_group(tree_file, out_group_txt, tree_file_rooted):
     tre.write(outfile=tree_file_rooted)
 
 
-def replace_clades(main_tree, sub_tree, tree_out):
+def replace_clades(main_tree, sub_tree, tree_out, quote_node_name):
 
-    tre_sub = Tree(sub_tree, format=1)
+    tre_sub = Tree(sub_tree, format=1, quoted_node_names=quote_node_name)
     subtree_leaf_name_list = tre_sub.get_leaf_names()
     tre_main = Tree(main_tree)
     lca = tre_main.get_common_ancestor(subtree_leaf_name_list)
@@ -58,17 +57,18 @@ def replace_clades(main_tree, sub_tree, tree_out):
     lca_p = lca.up
     lca_p.remove_child(lca)
     lca_p.add_child(tre_sub)
-    tre_main.write(outfile=tree_out, format=8)
+    tre_main.write(outfile=tree_out, format=8, quoted_node_names=quote_node_name)
 
 
 def prep_mcmctree_ctl(ctl_para_dict, mcmctree_ctl_file):
+
     with open(mcmctree_ctl_file, 'w') as ctl_file_handle:
-        ctl_file_handle.write('      finetune = %s\n' % ctl_para_dict.get('seed',         '-1'))
+        ctl_file_handle.write('      finetune = %s\n' % ctl_para_dict.get('seed', '-1'))
         ctl_file_handle.write('       seqfile = %s\n' % ctl_para_dict['seqfile'])
         ctl_file_handle.write('      treefile = %s\n' % ctl_para_dict['treefile'])
         ctl_file_handle.write('      mcmcfile = %s\n' % ctl_para_dict['mcmcfile'])
         ctl_file_handle.write('       outfile = %s\n' % ctl_para_dict['outfile'])
-        ctl_file_handle.write('         ndata = %s\n' % ctl_para_dict.get('ndata',        1))
+        ctl_file_handle.write('         ndata = %s\n'                                                                           % ctl_para_dict.get('ndata',        1))
         ctl_file_handle.write('       seqtype = %s    	* 0: nucleotides; 1:codons; 2:AAs\n'                                    % ctl_para_dict['seqtype'])
         ctl_file_handle.write('       usedata = %s    	* 0: no data; 1:seq like; 2:normal approximation; 3:out.BV (in.BV)\n'   % ctl_para_dict['usedata'])
         ctl_file_handle.write('         clock = %s    	* 1: global clock; 2: independent rates; 3: correlated rates\n'         % ctl_para_dict['clock'])
@@ -86,7 +86,7 @@ def prep_mcmctree_ctl(ctl_para_dict, mcmctree_ctl_file):
         ctl_file_handle.write('         print = %s      * 0: no mcmc sample; 1: everything except branch rates 2: everything\n' % ctl_para_dict.get('print',        1))
         ctl_file_handle.write('        burnin = %s\n'                                                                           % ctl_para_dict.get('burnin',       50000))
         ctl_file_handle.write('      sampfreq = %s\n'                                                                           % ctl_para_dict.get('sampfreq',     5))
-        ctl_file_handle.write('       nsample = %s\n'                                                                           % ctl_para_dict.get('nsample',      50000))
+        ctl_file_handle.write('       nsample = %s\n'                                                                           % ctl_para_dict.get('nsample',      150000))
 
 
 def get_parameter_combinations(para_to_test_dict):
@@ -126,6 +126,24 @@ def get_parameter_combinations(para_to_test_dict):
     return para_dod
 
 
+def fa2phy(fasta_in, phy_out):
+
+    alignment = AlignIO.read(fasta_in, 'fasta')
+
+    max_seq_id_len = 0
+    for each_seq in alignment:
+        seq_id_len = len(each_seq.id)
+        if seq_id_len > max_seq_id_len:
+            max_seq_id_len = seq_id_len
+
+    with open(phy_out, 'w') as msa_out_handle:
+        msa_out_handle.write('%s %s\n' % (len(alignment), alignment.get_alignment_length()))
+        for each_seq in alignment:
+            seq_id = each_seq.id
+            seq_id_with_space = '%s%s' % (seq_id, ' ' * (max_seq_id_len + 2 - len(seq_id)))
+            msa_out_handle.write('%s%s\n' % (seq_id_with_space, str(each_seq.seq)))
+
+
 def Dating(args):
 
     deltall_stdout_txt      = args['deltall']
@@ -135,10 +153,19 @@ def Dating(args):
     op_dir                  = args['o']
     deltall_keep_pct_str    = args['c']
     min_marker_num          = args['mmn']
-    js_cpu_num              = args['jst']
     force_overwrite         = args['f']
     root_age                = args['ra']
     submit_job              = args['qsub']
+    para_to_test            = args['to_test']
+    js_cpu_num              = 1
+    quote_node_name         = False
+
+    para_to_test_dict = dict()
+    for each_para in open(para_to_test):
+        each_para_split = each_para.strip().split()
+        para_list = each_para_split[1].split(',')
+        para_to_test_dict[each_para_split[0]] = para_list
+    print('Parameters to test: %s' % para_to_test_dict)
 
     if os.path.isfile(eu_tree) is False:
         print('%s not found, program exited!' % eu_tree)
@@ -209,33 +236,37 @@ def Dating(args):
         else:
             prefix_base                              = '%s_DeltaLL_%s'                                          % (deltall_stdout_basename.split('_DeltaLL_stdout')[0], each_keep_pct)
             aln_concatenated                         = '%s_DeltaLL_%s_concatenated.phy'                         % (deltall_stdout_basename.split('_DeltaLL_stdout')[0], each_keep_pct)
+            aln_concatenated_in_aod_wd_fasta         = '%s_DeltaLL_%s_concatenated.phy.fasta'                   % (deltall_stdout_basename.split('_DeltaLL_stdout')[0], each_keep_pct)
             c60_tree_file_rooted_with_time_final     = '%s_DeltaLL_%s_rooted_with_time_final.treefile'          % (deltall_stdout_basename.split('_DeltaLL_stdout')[0], each_keep_pct)
             pwd_c60_tree_file                        = '%s/%s_DeltaLL_%s_iqtree_C60_PMSF/concatenated.treefile' % (aod, deltall_stdout_basename.split('_DeltaLL_stdout')[0], each_keep_pct)
             pwd_c60_tree_file_renamed                = '%s/%s_DeltaLL_%s_raw.treefile'                          % (op_dir, deltall_stdout_basename.split('_DeltaLL_stdout')[0], each_keep_pct)
             pwd_c60_tree_file_rooted                 = '%s/%s_DeltaLL_%s_rooted.treefile'                       % (op_dir, deltall_stdout_basename.split('_DeltaLL_stdout')[0], each_keep_pct)
             pwd_c60_tree_file_rooted_with_time       = '%s/%s_DeltaLL_%s_rooted_with_time.treefile'             % (op_dir, deltall_stdout_basename.split('_DeltaLL_stdout')[0], each_keep_pct)
-            pwd_aln_concatenated                     = '%s/%s'                                                  % (aod, aln_concatenated)
+            pwd_aln_concatenated_in_aod_wd_fasta     = '%s/%s'                                                  % (aod, aln_concatenated_in_aod_wd_fasta)
+            pwd_aln_concatenated_in_op_wd_phylip     = '%s/%s'                                                  % (op_dir, aln_concatenated)
             pwd_c60_tree_file_rooted_with_time_final = '%s/%s'                                                  % (op_dir, c60_tree_file_rooted_with_time_final)
             get_BV_wd                                = '%s/%s_get_BV_wd'                                        % (op_dir, prefix_base)
+            pwd_aln_concatenated_in_bv_wd_phylip     = '%s/%s'                                                  % (get_BV_wd, aln_concatenated)
 
-            os.system('cp %s %s/' % (pwd_aln_concatenated, op_dir))
+            fa2phy(pwd_aln_concatenated_in_aod_wd_fasta, pwd_aln_concatenated_in_op_wd_phylip)
             os.system('cp %s %s'  % (pwd_c60_tree_file, pwd_c60_tree_file_renamed))
 
             # root genome tree with outgroup
             root_with_out_group(pwd_c60_tree_file_renamed, out_group_txt, pwd_c60_tree_file_rooted)
 
             # add time constraints
-            replace_clades(pwd_c60_tree_file_rooted, eu_tree, pwd_c60_tree_file_rooted_with_time)
+            replace_clades(pwd_c60_tree_file_rooted, eu_tree, pwd_c60_tree_file_rooted_with_time, quote_node_name)
 
             # remove "NoName" from the rooted tree with time constraints
             tree_str = open(pwd_c60_tree_file_rooted_with_time).readline().strip().replace('NoName', '')
 
             # add root age
             tree_str = tree_str.replace(';', '<%s;' % root_age)
-            tre_object = Tree(tree_str, format=1)
+            tre_object = Tree(tree_str, format=8, quoted_node_names=quote_node_name)
             with open(pwd_c60_tree_file_rooted_with_time_final, 'w') as pwd_c60_tree_file_rooted_with_time_final_hanlde:
                 pwd_c60_tree_file_rooted_with_time_final_hanlde.write('%s\t1\n' % len(tre_object.get_leaf_names()))
-                pwd_c60_tree_file_rooted_with_time_final_hanlde.write(tree_str + '\n')
+                pwd_c60_tree_file_rooted_with_time_final_hanlde.write(tree_str.replace('""', '') + '\n')
+                #pwd_c60_tree_file_rooted_with_time_final_hanlde.write(tree_str + '\n')
 
             # rm tmp tree files
             os.system('rm %s' % pwd_c60_tree_file_renamed)
@@ -244,7 +275,7 @@ def Dating(args):
 
             # get BV file
             os.mkdir(get_BV_wd)
-            os.system('cp %s %s/' % (pwd_aln_concatenated, get_BV_wd))
+            fa2phy(pwd_aln_concatenated_in_aod_wd_fasta, pwd_aln_concatenated_in_bv_wd_phylip)  # sequence in phylip format need to be in one line
             os.system('cp %s %s/' % (pwd_c60_tree_file_rooted_with_time_final, get_BV_wd))
 
             get_BV_js                       = '%s/%s_get_BV.sh'         % (op_dir, prefix_base)
@@ -262,24 +293,23 @@ def Dating(args):
             prep_mcmctree_ctl(get_BV_para_dict, pwd_get_BV_mcmctree_ctl)
 
             with open(get_BV_js, 'w') as get_BV_js_handle:
-                get_BV_js_handle.write('#!/bin/bash\n\n')
+                get_BV_js_handle.write('#!/bin/bash\n#SBATCH --ntasks 1\n#SBATCH --cpus-per-task %s\n\n' % js_cpu_num)
                 get_BV_js_handle.write('cd %s/%s\n' % (os.getcwd(), get_BV_wd))
                 get_BV_js_handle.write('mcmctree %s\n' % get_BV_mcmctree_ctl)
 
             # prepare files for dating
-            para_to_test_dict = {'clock': [2, 3], 'model': [0, 4]}
-            para_to_test_dict = {'clock': [2, 3]}
             para_dod = get_parameter_combinations(para_to_test_dict)
             for para_combination in para_dod:
-                mcmctree_ctl        = '%s_%s_mcmctree.ctl'              % (prefix_base, para_combination)
-                current_dating_wd   = '%s/%s_DeltaLL_%s_%s_dating_wd'   % (op_dir, deltall_stdout_basename.split('_DeltaLL_stdout')[0], each_keep_pct, para_combination)
-                pwd_mcmctree_ctl    = '%s/%s_%s_mcmctree.ctl'           % (current_dating_wd, prefix_base, para_combination)
-                js_mcmctree         = '%s/%s_DeltaLL_%s_%s.sh'          % (op_dir, deltall_stdout_basename.split('_DeltaLL_stdout')[0], each_keep_pct, para_combination)
+                mcmctree_ctl         = '%s_%s_mcmctree.ctl'             % (prefix_base, para_combination)
+                current_dating_wd    = '%s/%s_DeltaLL_%s_%s_dating_wd'  % (op_dir, deltall_stdout_basename.split('_DeltaLL_stdout')[0], each_keep_pct, para_combination)
+                pwd_mcmctree_ctl     = '%s/%s_%s_mcmctree.ctl'          % (current_dating_wd, prefix_base, para_combination)
+                js_mcmctree          = '%s/js_%s_DeltaLL_%s_%s.sh'      % (op_dir, deltall_stdout_basename.split('_DeltaLL_stdout')[0], each_keep_pct, para_combination)
+                pwd_aln_in_dating_wd = '%s/%s'                          % (current_dating_wd, aln_concatenated)
 
                 # create dating wd and copy tree and alignment files into it
                 os.mkdir(current_dating_wd)
-                os.system('cp %s %s/'       % (pwd_aln_concatenated, current_dating_wd))
-                os.system('cp %s %s/'       % (pwd_c60_tree_file_rooted_with_time_final, current_dating_wd))
+                fa2phy(pwd_aln_concatenated_in_aod_wd_fasta, pwd_aln_in_dating_wd)  # sequence in phylip format need to be in one line
+                os.system('cp %s %s/' % (pwd_c60_tree_file_rooted_with_time_final, current_dating_wd))
 
                 current_para_dict = para_dod[para_combination]
                 current_para_dict['seqfile']  = aln_concatenated
@@ -304,19 +334,18 @@ def Dating(args):
 
 if __name__ == '__main__':
 
-    # initialize the options parser
     parser = argparse.ArgumentParser()
     parser.add_argument('-deltall', required=True,                          help='DeltaLL stdout')
     parser.add_argument('-aod',     required=True,                          help='AssessMarkerDeltaLL output dir')
-    parser.add_argument('-og',      required=True,                          help='outgroup leaves, one leaf id per line')
+    parser.add_argument('-og',      required=True,                          help='outgroup leaves, one id per line')
     parser.add_argument('-eu',      required=True,                          help='EU tree with time constraints')
     parser.add_argument('-o',       required=True,                          help='dating wd')
     parser.add_argument('-c',       required=False, default='25-50-75-100', help='cutoffs, default: 25-50-75-100')
     parser.add_argument('-mmn',     required=False, default=20, type=int,   help='minimal marker number, default: 20')
     parser.add_argument('-ra',      required=False, default=45, type=int,   help='root age, default: 45')
-    parser.add_argument('-jst',     required=False, default='6',            help='threads to request in job script, for performing dating')
     parser.add_argument('-qsub',    required=False, action="store_true",    help='submit job scripts for getting in.BV')
     parser.add_argument('-f',       required=False, action="store_true",    help='force overwrite')
+    parser.add_argument('-to_test', required=True,                          help='Settings to test')
     args = vars(parser.parse_args())
     Dating(args)
 
